// JavaScript

import {Contract, Provider} from "zksync-web3";
import {defaultAbiCoder} from "ethers/lib/utils.js";
import {BigNumber} from "ethers";
import poolFactoryAbi from "./abi/poolFactoryAbi.js";
import routerAbi from "./abi/routerAbi.js";


const zksrpc = 'https://mainnet.era.zksync.io';
const provider = new Provider(zksrpc);

// 设置代币地址
const wETHAddress = '0x5aea5775959fbc2557cc8789bc1bf90a239d9a91';
const usdcAddress = '0x3355df6D4c9C3035724Fd0e3914dE96A5a83aaf4';
const ZERO_ADDRESS = '0x0000000000000000000000000000000000000000';
const ETH_ADDRESS = '0x0000000000000000000000000000000000000000';

// 设置合约地址
const factoryAddress = '0xf2DAd89f2788a8CD54625C60b55cD3d2D0ACa7Cb';
const routerAddress = '0x2da10A1e27bF85cEdD8FFb1AbBe97e53391C0295';

export {
    wETHAddress,
    usdcAddress,
    ZERO_ADDRESS,
    ETH_ADDRESS,
    routerAddress
}

export async function getPoolAddress(tokenA, tokenB) {
    const poolFactory = new Contract(factoryAddress, poolFactoryAbi, provider);
    return await poolFactory.getPool(tokenA, tokenB);
}

export async function tokenSwap(poolAddress, tokenIn, amountIn, wallet) {


    // 交易模式：
    // 1 - 提取并解包装到原生ETH
    // 2 - 提取并包装到wETH
    const withdrawMode = 1;
    // console.log(wallet.address);
    // process.exit();
    // wallet.getSigner(0)
    // console.log("address",wallet.address)
    // 构建交易参数
    const swapData = defaultAbiCoder.encode(
        ["address", "address", "uint8"],
        [tokenIn, wallet.address, withdrawMode]
    );
    const steps = [{
        pool: poolAddress,
        data: swapData,
        callback: ZERO_ADDRESS, // we don't have a callback
        callbackData: '0x',
    }];

    if (tokenIn === wETHAddress) {
        tokenIn = ZERO_ADDRESS;
    }
    const paths = [{
        steps: steps,
        tokenIn: tokenIn,
        amountIn: amountIn,
    }];

    // 创建router合约
    const router = new Contract(routerAddress, routerAbi, wallet);


    // params
    const params = {};
    // 使用原生ETH支付时，需要传入value参数
    if (paths[0].tokenIn === ZERO_ADDRESS) {
        params.value = amountIn
    }
    // 获取gas价格
    params.gasPrice = await provider.getGasPrice();
    // 估计交易所需的 gas limit
    params.gasLimit = await router.estimateGas.swap(paths, 0, BigNumber.from(Math.floor(Date.now() / 1000)).add(1800), params);

    // 开始兑换
    const response = await router.swap(
        paths,
        0,
        BigNumber.from(Math.floor(Date.now() / 1000)).add(1800),
        params
    );
    // console.log(response)
    return await response.wait();
}