import express from "express";
import mathUtil from "../../utils/mathUtil.js";
import {Provider, Wallet} from "zksync-web3";
import {tokenSwap, getPoolAddress, wETHAddress, usdcAddress} from "./swap.js";
import {utils} from "ethers";

const router = express.Router();
router.post("/", async (req, res) => {
    const params = req.body.params;
    const amountMin = params.amountMin;
    const amountMax = params.amountMax;
    const accuracy = params.accuracy;
    const amountNumber = mathUtil.getRandomNumber(amountMin, amountMax, accuracy);
    const signer = req.body.wallet;
    const provider = new Provider('https://mainnet.era.zksync.io');
    const wallet = new Wallet(signer.privateKey, provider);
    const pool = await getPoolAddress(wETHAddress, usdcAddress);
    const txR = await tokenSwap(pool, wETHAddress, utils.parseEther(`${amountNumber}`), wallet);
    const tx = await provider.getTransaction(txR.transactionHash);
    // const swapReceipt = await provider.getTransactionReceipt(tx.hash);
    const gas = utils.formatEther((txR.gasUsed * tx.gasPrice).toFixed(0));
    const balance = await wallet.getBalance()
    const balanceStr = utils.formatEther(balance);
    const value = utils.formatEther(tx.value)
    res.send({
        code: 0,
        msg: "",
        data: {
            txId: tx.hash,
            gas: `${parseFloat(gas)}`,
            currentBalance: balanceStr,
            status: txR.status,
            value: value
        }
    })

})
export default router;